#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Download, unzip and prep the latest CAL-ACCESS database ZIP.
"""
from __future__ import unicode_literals
import os
import shutil
import zipfile
import requests
import calaccess_raw
from datetime import datetime
from hurry.filesize import size
from clint.textui import progress
from django.conf import settings
from django.core.files import File
from django.utils.timezone import utc, now
from calaccess_raw import get_download_directory
from django.template.loader import render_to_string
from django.core.management.base import CommandError
from calaccess_raw.management.commands import CalAccessCommand
from django.contrib.humanize.templatetags.humanize import naturaltime
from calaccess_raw.models.tracking import RawDataVersion, RawDataFile


class Command(CalAccessCommand):
    """
    Download, unzip and prep the latest CAL-ACCESS database ZIP.
    """
    help = "Download, unzip and prep the latest CAL-ACCESS database ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep downloaded zip and unzipped files"
        )
        parser.add_argument(
            "--noinput",
            action="store_true",
            dest="noinput",
            default=False,
            help="Download the ZIP archive without asking permission"
        )
        parser.add_argument(
            "--force-restart",
            "--restart",
            action="store_true",
            dest="restart",
            default=False,
            help="Force re-start (overrides auto-resume)."
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # get the dir where data goes from app settings
        self.data_dir = get_download_directory()
        # if data_dir doesn't exist, create it
        os.path.exists(self.data_dir) or os.makedirs(self.data_dir)

        # downloaded zip file will go in data_dir
        self.zip_path = os.path.join(self.data_dir, self.url.split('/')[-1])
        # raw tsv files go in same data_dir in tsv/
        self.tsv_dir = os.path.join(self.data_dir, "tsv/")

        download_metadata = self.get_download_metadata()

        self.current_release_datetime = download_metadata['last-modified']
        self.current_release_size = download_metadata['content-length']

        self.last_started_download = self.get_last_log()
        self.last_finished_download = self.get_last_log(finished=True)

        if self.last_finished_download:
            last_release_datetime = self.last_finished_download.version.release_datetime
            since_prev_version = naturaltime(last_release_datetime)
        else:
            last_release_datetime = None
            since_prev_version = None

        if last_release_datetime == self.current_release_datetime:
            already_downloaded = True
        else:
            already_downloaded = False

        # can resume only if possible and not forcing re-start
        self.resume_download = self.check_can_resume() and not options['restart']

        if self.resume_download:
            # set current size to partially downloaded zip
            self.local_file_size = os.path.getsize(self.zip_path)
            # set the datetime of last download to last modified date
            # of zip file
            timestamp = os.path.getmtime(self.zip_path)
            self.local_file_datetime = datetime.fromtimestamp(timestamp, utc)
        else:
            self.local_file_size = 0
            self.local_file_datetime = None

        if not options['noinput'] and not options['restart']:

            # setting up the prompt
            prompt_context = dict(
                current_release_datetime=self.current_release_datetime,
                resuming=self.resume_download,
                already_downloaded=already_downloaded,
                expected_size=size(self.current_release_size),
                local_file_size=size(self.local_file_size),
                download_dir=self.data_dir,
                since_prev_version=since_prev_version,
                since_local_file_modified=naturaltime(self.local_file_datetime)
            )

            prompt = render_to_string(
                'calaccess_raw/downloadcalaccessrawdata.txt',
                prompt_context,
            )

            if not self.confirm_proceed(prompt):
                raise CommandError("Download cancelled")

        if self.resume_download:
            self.log_record = self.last_started_download
            self.version = self.log_record.version
        else:
            # get or create a version record
            # .get_or_create() throws IntegrityError
            try:
                self.version = self.raw_data_versions.get(
                    release_datetime=self.current_release_datetime
                )
            except RawDataVersion.DoesNotExist:
                self.version = self.raw_data_versions.create(
                    release_datetime=self.current_release_datetime,
                    size=download_metadata['content-length']
                )
            # create a log record
            self.log_record = self.command_logs.create(
                version=self.version,
                command=self,
                called_by=self.get_caller_log()
            )

        self.download()
        self.unzip()
        self.prep()
        self.track_files()

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.archive()

        if not options['keep_files']:
            os.remove(self.zip_path)
            shutil.rmtree(os.path.join(self.data_dir, 'CalAccess'))

        self.log_record.finish_datetime = now()
        self.log_record.save()

    def check_can_resume(self):
        """
        Run a series of checks to see if the previous download can be resumed.

        If so, return True, else False.
        """
        result = False
        # if there's a zip file
        if os.path.exists(self.zip_path):
            # and there's a previous download
            if self.last_started_download:
                # that did not finish
                if not self.last_started_download.finish_datetime:

                    prev_release = self.last_started_download.version.release_datetime

                    # and the current release datetime is the same as
                    #  the one on the last incomplete download
                    if self.current_release_datetime == prev_release:
                        result = True
        return result

    def download(self):
        """
        Download the ZIP file in pieces.
        """
        if self.verbosity:
            if self.resume_download:
                self.header("Resuming download of ZIP file")
            else:
                self.header("Downloading ZIP file")

        expected_size = self.current_release_size

        # Prep
        headers = dict()
        if os.path.exists(self.zip_path):
            if self.resume_download:
                headers['Range'] = 'bytes=%d-' % self.local_file_size
                expected_size = expected_size - self.local_file_size
            else:
                os.remove(self.zip_path)

        # Stream the download
        chunk_size = 1024
        req = requests.get(self.url, stream=True, headers=headers)
        n_iters = float(expected_size) / chunk_size + 1
        with open(self.zip_path, 'ab') as fp:
            for chunk in progress.bar(req.iter_content(chunk_size=chunk_size),
                                      expected_size=n_iters):
                fp.write(chunk)
                fp.flush()

    def unzip(self):
        """
        Unzip the snapshot file.
        """
        if self.verbosity:
            self.log(" Unzipping archive")

        with zipfile.ZipFile(self.zip_path) as zf:
            for member in zf.infolist():
                words = member.filename.split('/')
                path = self.data_dir
                for word in words[:-1]:
                    drive, word = os.path.splitdrive(word)
                    head, word = os.path.split(word)
                    if word in (os.curdir, os.pardir, ''):
                        continue
                    path = os.path.join(path, word)
                zf.extract(member, path)

    def prep(self):
        """
        Rearrange the unzipped files and get rid of the stuff we don't want.
        """
        if self.verbosity:
            self.log(" Prepping unzipped data")

        # Move the deep down directory we want out
        shutil.move(
            os.path.join(
                self.data_dir,
                'CalAccess/DATA/CalAccess/DATA/'
            ),
            self.data_dir
        )
        # Clear out target if it exists
        if os.path.exists(self.tsv_dir):
            shutil.rmtree(self.tsv_dir)

        # Rename it to the target
        shutil.move(
            os.path.join(self.data_dir, "DATA/"),
            self.tsv_dir,
        )

    def track_files(self):
        """
        Create a RawDataFile for each download .TSV file.
        """
        for f in os.listdir(self.tsv_dir):
            if '.TSV' in f:
                file_name = f.upper().replace('.TSV', '')
                self.raw_data_files.get_or_create(
                    version=self.version,
                    file_name=file_name,
                )

    def archive(self):
        """
        Save a copy of the download zip file and each file inside.
        """
        if self.verbosity:
            self.log(" Archiving original files")
        # Remove previous zip file
        self.version.zip_file_archive.delete()
        # Open up the zipped file so we can wrap it in the Django File obj
        zipped_file = open(self.zip_path)
        # Save the zip on the raw data version
        self.version.zip_file_archive.save(
            self.url.split('/')[-1],
            File(zipped_file)
        )
        zipped_file.close()

        # make the RawDataFile records
        for raw_data_file in self.raw_data_files.filter(version=self.version):
            # Remove previous .TSV file
            raw_data_file.download_file_archive.delete()
            # Open up the .TSV file so we can wrap it in the Django File obj
            with open(self.tsv_dir + raw_data_file.file_name + '.TSV') as f:
                # Save the .TSV on the raw data file
                raw_data_file.download_file_archive.save(
                    raw_data_file.file_name + '.TSV',
                    File(f)
                )
                if self.verbosity > 2:
                    self.log(" Archiving {0}.TSV".format(raw_data_file.file_name)


class TestCommand(Command):
    """
    Simulates downloading and unzipping of CAL-ACCESS raw data for testing.
    """
    help = "Simulates downloading and unzipping of CAL-ACCESS raw data for testing"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(TestCommand, self).add_arguments(parser)

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        self.verbosity = options.get("verbosity")
        self.no_color = options.get("no_color")
        self.raw_data_files = RawDataFile.objects
        self.data_dir = calaccess_raw.get_test_download_directory()
        self.tsv_dir = os.path.join(self.data_dir, "tsv/")
        self.zip_path = os.path.join(self.data_dir, self.url.split('/')[-1])

        with open(self.data_dir + "/sampled_version.txt", "r") as f:
            release_datetime = f.readline()
            size = f.readline()

        try:
            self.version = RawDataVersion.objects.get(
                release_datetime=release_datetime
            )
        except RawDataVersion.DoesNotExist:
            self.version = RawDataVersion.objects.create(
                release_datetime=release_datetime,
                size=size
            )

        self.unzip()
        self.prep()
        self.track_files()

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.archive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for tracking CAL-ACCESS updates over time.
"""
from __future__ import unicode_literals
from django.db import models
from hurry.filesize import size as sizeformat
from calaccess_raw import archive_directory_path, get_model_list
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class RawDataVersion(models.Model):
    """
    A version of CAL-ACCESS raw source data, typically released every day.
    """
    release_datetime = models.DateTimeField(
        unique=True,
        null=False,
        verbose_name='date and time of release',
        help_text='Date and time the version of the CAL-ACCESS database was released '
                  '(value of last-modified field in HTTP response header)'
    )
    size = models.BigIntegerField(
        null=False,
        verbose_name='size of raw data version in bytes',
        help_text='Size of the .ZIP file for this version of the CAL-ACCESS raw source data '
                  '(value of content-length field in HTTP response header)'
    )
    zip_file_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of zip file',
        help_text='An archive of the original zipped file downloaded from '
                     'CAL-ACCESS.'
    )

    class Meta:
        app_label = 'calaccess_raw'
        verbose_name = 'CAL-ACCESS raw data version'
        ordering = ('-release_datetime',)

    def __str__(self):
        return str(self.release_datetime)

    def pretty_size(self):
        """
        Returns a prettified version of the file size.
        """
        if not self.size:
            return None
        return sizeformat(self.size)
    pretty_size.short_description = 'size'
    pretty_size.admin_order_field = 'size'


@python_2_unicode_compatible
class RawDataFile(models.Model):
    """
    A data file included in a version of the CAL-ACCESS raw source data.
    """
    version = models.ForeignKey(
        'RawDataVersion',
        on_delete=models.CASCADE,
        related_name='files',
        verbose_name='raw data version',
        help_text='Foreign key referencing the version of the raw '
                  'source data in which the file was included'
    )
    file_name = models.CharField(
        max_length=100,
        verbose_name='raw data file name',
        help_text='Name of the raw source data file without extension',
    )
    download_records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='download records count',
        help_text='Count of records in the original file downloaded from CAL-ACCESS'
    )
    clean_records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='clean records count',
        help_text='Count of records in the cleaned file generated by calaccess_raw'
    )
    load_records_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='load records count',
        help_text="Count of records in the loaded from cleaned file into "
                  "calaccess_raw's data model"
    )
    download_columns_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='download columns count',
        help_text='Count of columns in the original file downloaded from CAL-ACCESS'
    )
    clean_columns_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='clean columns count',
        help_text='Count of columns in the cleaned file generated by calaccess_raw'
    )
    load_columns_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='load columns count',
        help_text='Count of columns on the loaded calaccess_raw data model'
    )
    download_file_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of download file',
        help_text='An archive of the original raw data file downloaded '
                     'from CAL-ACCESS.'
    )
    download_file_size = models.BigIntegerField(
        null=False,
        default=0,
        verbose_name='size of raw data file in bytes',
        help_text='Size of the .TSV file'
    )
    clean_file_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of clean file',
        help_text='An archive of the raw data file after being cleaned.'
    )
    clean_file_size = models.BigIntegerField(
        null=False,
        default=0,
        verbose_name='size of clean data file in bytes',
        help_text='Size of the .CSV file'
    )
    error_count = models.IntegerField(
        null=False,
        default=0,
        verbose_name='error count',
        help_text='Count of records in the original download that could not '
                  'be parsed and are excluded from the cleaned file.'
    )
    error_log_archive = models.FileField(
        blank=True,
        max_length=255,
        upload_to=archive_directory_path,
        verbose_name='archive of error log',
        help_text='An archive of the error log containing lines from the '
                  'original download file that could not be parsed and are '
                  'excluded from the cleaned file.'
    )

    class Meta:
        app_label = 'calaccess_raw'
        unique_together = (('version', 'file_name'),)
        verbose_name = 'CAL-ACCESS raw data file'
        ordering = ('-version_id', 'file_name',)

    def __str__(self):
        return self.file_name

    def pretty_download_file_size(self):
        """
        Returns a prettified version of the download_file_size size.
        """
        if not self.download_file_size:
            return None
        return sizeformat(self.download_file_size)
    pretty_download_file_size.short_description = 'download file size'
    pretty_download_file_size.admin_order_field = 'download file size'

    def pretty_clean_file_size(self):
        """
        Returns a prettified version of the clean_file_size size.
        """
        if not self.clean_file_size:
            return None
        return sizeformat(self.clean_file_size)
    pretty_clean_file_size.short_description = 'clean file size'
    pretty_clean_file_size.admin_order_field = 'clean file size'

    @property
    def model(self):
        """
        Returns the RawDataFile's corresponding CalAccess database model object.
        """
        return [
            m for m in get_model_list() if m().db_table == self.file_name
        ][0]


@python_2_unicode_compatible
class RawDataCommand(models.Model):
    """
    A call of a command from this application with start and finish times.
    """
    version = models.ForeignKey(
        'RawDataVersion',
        on_delete=models.CASCADE,
        related_name='command_logs',
        verbose_name='raw data version',
        help_text='Foreign key referencing the version of the raw '
                  'source data on which the command was performed'
    )
    command = models.CharField(
        max_length=50,
        verbose_name='command name',
        help_text='Name of the command performed on the given version of the raw source data'
    )
    called_by = models.ForeignKey(
        'self',
        related_name="called",
        null=True,
        on_delete=models.SET_NULL,
        verbose_name='called by',
        help_text='Foreign key refencing log of the CalAccessCommand that '
                  'called this command. Null represents call from command line'
    )
    file_name = models.CharField(
        max_length=100,
        verbose_name='raw data file name',
        help_text='Name of the raw source data file without extension',
    )
    start_datetime = models.DateTimeField(
        auto_now_add=True,
        null=False,
        verbose_name='date and time command started',
        help_text='Date and time when the given command started on the '
                  'given version of the raw source data'
    )
    finish_datetime = models.DateTimeField(
        null=True,
        verbose_name='date and time command finished',
        help_text='Date and time when the given command finished on '
                  'the given version of the raw source data'
    )

    class Meta:
        app_label = 'calaccess_raw'
        verbose_name = 'CAL-ACCESS raw data command'
        ordering = ('-id',)

    def __str__(self):
        return self.command



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-07 05:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0006_rawdatafile_error_log_archive'),
    ]

    operations = [
        migrations.AddField(
            model_name='rawdatafile',
            name='error_count',
            field=models.IntegerField(default=0, help_text='Count of records in the original download that could not be parsed and are excluded from the cleaned file.', verbose_name='error count'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for lobbying models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_raw import models
from .base import BaseAdmin


@admin.register(models.CvrRegistrationCd)
class CvrRegistrationCdAdmin(BaseAdmin):
    """
    Custom admin for the CvrRegistrationCd model.
    """
    list_display = (
        "filing_id",
        "rpt_date",
        "form_type",
        "filer_naml",
        "firm_name",
    )
    date_hierarchy = "rpt_date"


@admin.register(models.Cvr2RegistrationCd)
class Cvr2RegistrationCdAdmin(BaseAdmin):
    """
    Custom admin for the Cvr2RegistrationCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "enty_naml",
    )


@admin.register(models.LobbyAmendmentsCd)
class LobbyAmendmentsCdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyAmendmentsCd model.
    """
    list_display = (
        "filing_id",
        "exec_date",
        "form_type",
    )
    date_hierarchy = "exec_date"


@admin.register(models.LobbyingChgLogCd)
class LobbyingChgLogCdAdmin(BaseAdmin):
    """
    Custom admin for the LobbyingChgLogCd model.
    """
    list_display = (
        "filer_id",
        "log_dt",
        "filer_full_name",
    )
    date_hierarchy = "log_dt"


@admin.register(models.LempCd)
class LempCdAdmin(BaseAdmin):
    """
    Custom admin for the LempCd model.
    """
    list_display = (
        "filing_id",
        "eff_date",
        "cli_naml",
        "form_type",
    )


@admin.register(models.F690P2Cd)
class F690P2CdAdmin(BaseAdmin):
    """
    Custom admin for the F690P2Cd model.
    """
    list_display = (
        "filing_id",
        "exec_date",
        "form_type",
    )
    date_hierarchy = "exec_date"


@admin.register(models.CvrLobbyDisclosureCd)
class CvrLobbyDisclosureCdAdmin(BaseAdmin):
    """
    Custom admin for the CvrLobbyDisclosureCd model.
    """
    list_display = (
        "filing_id",
        "rpt_date",
        "filer_naml",
        "firm_name",
        "form_type",
    )
    date_hierarchy = "rpt_date"


@admin.register(models.Cvr2LobbyDisclosureCd)
class Cvr2LobbyDisclosureCdAdmin(BaseAdmin):
    """
    Custom admin for the Cvr2LobbyDisclosureCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "entity_cd",
        "enty_naml",
    )


@admin.register(models.LattCd)
class LattCdAdmin(BaseAdmin):
    """
    Custom admin for the LattCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "pmt_date",
        "recip_naml",
        "amount"
    )
    date_hierarchy = "pmt_date"


@admin.register(models.LexpCd)
class LexpCdAdmin(BaseAdmin):
    """
    Custom admin for the LexpCd model.
    """
    list_display = (
        "filing_id",
        "expn_date",
        "form_type",
        "payee_naml",
        "amount"
    )
    date_hierarchy = "expn_date"


@admin.register(models.LccmCd)
class LccmCdAdmin(BaseAdmin):
    """
    Custom admin for the LccmCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "ctrib_date",
        "ctrib_naml",
        "recip_naml",
    )
    date_hierarchy = "ctrib_date"


@admin.register(models.LpayCd)
class LpayCdAdmin(BaseAdmin):
    """
    Custom admin for the LpayCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "emplr_naml",
        "per_total"
    )


@admin.register(models.LothCd)
class LothCdAdmin(BaseAdmin):
    """
    Custom admin for the LothCd model.
    """
    list_display = (
        "filing_id",
        "pmt_date",
        "form_type",
        "firm_name",
        "amount"
    )
    date_hierarchy = "pmt_date"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import all of the models from submodules and thread them together.
"""
from calaccess_raw.models.base import CalAccessBaseModel
from calaccess_raw.models.campaign import (
    CvrSoCd,
    Cvr2SoCd,
    CvrCampaignDisclosureCd,
    Cvr2CampaignDisclosureCd,
    Cvr3VerificationInfoCd,
    DebtCd,
    ExpnCd,
    LoanCd,
    RcptCd,
    S401Cd,
    F495P2Cd,
    S496Cd,
    S497Cd,
    S498Cd,
    F501502Cd,
)
from calaccess_raw.models.inactive import (
    BallotMeasuresCd,
    CvrF470Cd,
    FilerTypePeriodsCd,
    LobbyistContributions1Cd,
    LobbyistContributions2Cd,
    LobbyistContributions3Cd,
    LobbyistEmpLobbyist1Cd,
    LobbyistEmpLobbyist2Cd,
    LobbyistEmployer1Cd,
    LobbyistEmployer2Cd,
    LobbyistEmployer3Cd,
    LobbyistEmployerFirms1Cd,
    LobbyistEmployerFirms2Cd,
    LobbyistEmployerHistoryCd,
    LobbyistFirm1Cd,
    LobbyistFirm2Cd,
    LobbyistFirm3Cd,
    LobbyistFirmEmployer1Cd,
    LobbyistFirmEmployer2Cd,
    LobbyistFirmHistoryCd,
    LobbyistFirmLobbyist1Cd,
    LobbyistFirmLobbyist2Cd,
    EfsFilingLogCd,
)
from calaccess_raw.models.lobbying import (
    CvrRegistrationCd,
    Cvr2RegistrationCd,
    LobbyAmendmentsCd,
    LobbyingChgLogCd,
    LempCd,
    CvrLobbyDisclosureCd,
    Cvr2LobbyDisclosureCd,
    F690P2Cd,
    LattCd,
    LexpCd,
    LccmCd,
    LpayCd,
    LothCd,
)
from calaccess_raw.models.common import (
    CvrE530Cd,
    FilernameCd,
    FilerFilingsCd,
    FilingsCd,
    HdrCd,
    HeaderCd,
    SmryCd,
    SpltCd,
    TextMemoCd,
    AcronymsCd,
    AddressCd,
    FilersCd,
    FilerAcronymsCd,
    FilerAddressCd,
    FilerEthicsClassCd,
    FilerInterestsCd,
    FilerLinksCd,
    FilerStatusTypesCd,
    FilerToFilerTypeCd,
    FilerTypesCd,
    FilerXrefCd,
    FilingPeriodCd,
    GroupTypesCd,
    ImageLinksCd,
    LegislativeSessionsCd,
    LookupCodesCd,
    NamesCd,
    ReceivedFilingsCd,
    ReportsCd,
)
from calaccess_raw.models.tracking import (
    RawDataVersion,
    RawDataCommand,
    RawDataFile
)
__all__ = (
    'CalAccessBaseModel',
    'CvrSoCd',
    'Cvr2SoCd',
    'CvrCampaignDisclosureCd',
    'Cvr2CampaignDisclosureCd',
    'CvrF470Cd',
    'RcptCd',
    'Cvr3VerificationInfoCd',
    'LoanCd',
    'S401Cd',
    'ExpnCd',
    'F495P2Cd',
    'DebtCd',
    'S496Cd',
    'SpltCd',
    'S497Cd',
    'F501502Cd',
    'S498Cd',
    'CvrRegistrationCd',
    'Cvr2RegistrationCd',
    'CvrLobbyDisclosureCd',
    'Cvr2LobbyDisclosureCd',
    'LobbyAmendmentsCd',
    'F690P2Cd',
    'LattCd',
    'LexpCd',
    'LccmCd',
    'LothCd',
    'LempCd',
    'LpayCd',
    'FilerFilingsCd',
    'FilingsCd',
    'SmryCd',
    'CvrE530Cd',
    'TextMemoCd',
    'AcronymsCd',
    'AddressCd',
    'BallotMeasuresCd',
    'EfsFilingLogCd',
    'FilernameCd',
    'FilersCd',
    'FilerAcronymsCd',
    'FilerAddressCd',
    'FilerEthicsClassCd',
    'FilerInterestsCd',
    'FilerLinksCd',
    'FilerStatusTypesCd',
    'FilerToFilerTypeCd',
    'FilerTypesCd',
    'FilerXrefCd',
    'FilingPeriodCd',
    'FilerTypePeriodsCd',
    'GroupTypesCd',
    'HeaderCd',
    'HdrCd',
    'ImageLinksCd',
    'LegislativeSessionsCd',
    'LobbyingChgLogCd',
    'LobbyistContributions1Cd',
    'LobbyistContributions2Cd',
    'LobbyistContributions3Cd',
    'LobbyistEmployer1Cd',
    'LobbyistEmployer2Cd',
    'LobbyistEmployer3Cd',
    'LobbyistEmployerHistoryCd',
    'LobbyistEmployerFirms1Cd',
    'LobbyistEmployerFirms2Cd',
    'LobbyistEmpLobbyist1Cd',
    'LobbyistEmpLobbyist2Cd',
    'LobbyistFirm1Cd',
    'LobbyistFirm2Cd',
    'LobbyistFirm3Cd',
    'LobbyistFirmEmployer1Cd',
    'LobbyistFirmEmployer2Cd',
    'LobbyistFirmLobbyist1Cd',
    'LobbyistFirmLobbyist2Cd',
    'LobbyistFirmHistoryCd',
    'LookupCodesCd',
    'NamesCd',
    'ReceivedFilingsCd',
    'ReportsCd',
    'RawDataVersion',
    'RawDataCommand',
    'RawDataFile'
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base management command that provides common functionality for the other commands in this app.
"""
import sys
import codecs
import locale
import logging
import requests
from re import sub
from datetime import datetime
from email.utils import parsedate
from django.utils import timezone
from django.utils.six.moves import input
from django.utils.termcolors import colorize
from django.core.management.base import BaseCommand
from calaccess_raw.models.tracking import (
    RawDataVersion,
    RawDataFile,
    RawDataCommand
)
logger = logging.getLogger(__name__)


class CalAccessCommand(BaseCommand):
    """
    Base management command that provides common functionality for the other commands in this app.
    """
    url = 'http://campaignfinance.cdn.sos.ca.gov/dbwebexport.zip'

    def handle(self, *args, **options):
        """
        Sets options common to all commands.

        Any command subclassing this object should implement its own
        handle method, as is standard in Django, and run this method
        via a super call to inherit its functionality.
        """
        # Set global options
        self.verbosity = options.get("verbosity")
        self.no_color = options.get("no_color")

        # Start the clock
        self.start_datetime = datetime.now()

        # Set the logger models for later use
        self.raw_data_versions = RawDataVersion.objects
        self.raw_data_files = RawDataFile.objects
        self.command_logs = RawDataCommand.objects

    def get_download_metadata(self):
        """
        Returns basic metadata about the current CAL-ACCESS snapshot.
        """
        request = requests.head(self.url)
        last_modified = request.headers['last-modified']
        dt = datetime(*parsedate(last_modified)[:6])
        return {
            'content-length': int(request.headers['content-length']),
            'last-modified': timezone.utc.localize(dt)
        }

    def get_last_log(self, file_name=None, finished=False):
        """
        Retrives the given command most recently from database log.

        Unless finished=True, in which case query for the most recently finished.

        Commands that require a file / model as a positional argument should
        pass the file_name keyword argument.

        Returns a CalAccessCommandLog object or None, if no results.
        """
        if file_name:
            q = self.command_logs.filter(file_name=file_name)
        else:
            q = self.command_logs

        if finished:
            order_by_field = '-finish_datetime'
            q = q.filter(finish_datetime__isnull=False)
        else:
            order_by_field = '-start_datetime'

        try:
            last_log = q.filter(
                command=self
            ).order_by(order_by_field)[0]
        except IndexError:
            last_log = None

        return last_log

    def get_caller_log(self):
        """
        If the command was called by another command, returns RawDataCommandLog object.

        Otherwise, return None.
        """
        caller = None

        if not self._called_from_command_line:
            # TODO: see if there's another way to identify caller
            # in (edge) case when update is not called from command line

            # for now, assume the caller is the arg passed to manage.py
            # try getting the most recent log of this command for the version
            try:
                caller = self.command_logs.filter(
                    command=sys.argv[1]
                ).order_by('-start_datetime')[0]
            except IndexError:
                pass

        return caller

    #
    # Logging methods
    #

    def header(self, string):
        """
        Writes out a string to stdout formatted to look like a header.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize(string, fg="cyan", opts=("bold",))
        self.stdout.write(string)

    def log(self, string):
        """
        Writes out a string to stdout formatted to look like a standard line.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize("%s" % string, fg="white")
        self.stdout.write(string)

    def success(self, string):
        """
        Writes out a string to stdout formatted green to communicate success.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize(string, fg="green")
        self.stdout.write(string)

    def failure(self, string):
        """
        Writes out a string to stdout formatted red to communicate failure.
        """
        logger.debug(string)
        if not self.no_color:
            string = colorize(string, fg="red")
        self.stdout.write(string)

    def duration(self):
        """
        Calculates how long the command has been running and writes it to stdout.
        """
        duration = datetime.now() - self.start_datetime
        self.stdout.write('Duration: {}'.format(str(duration)))

    def confirm_proceed(self, prompt):
        """
        Prompts the user for yes/no confirmation to proceed.
        """
        # Ensure stdout can handle Unicode data: http://bit.ly/1C3l4eV
        locale_encoding = locale.getpreferredencoding()
        old_stdout = sys.stdout
        sys.stdout = codecs.getwriter(locale_encoding)(sys.stdout)

        # Send the confirmation prompt out to the user
        user_input = input(prompt)

        confirm = None

        while confirm is None:
            if user_input.lower() in ['y', 'yes']:
                confirm = True
            elif user_input.lower() in ['n', 'no']:
                confirm = False
            else:
                user_input = input("Invalid input. Please type 'yes', 'no', 'y' or 'n':\n")

        # Set things back to the way they were before continuing.
        sys.stdout = old_stdout

        # Pass back what the user typed
        return confirm

    def __str__(self):
        return sub(r'(.+\.)*', '', self.__class__.__module__)
